package org.jeecg.modules.ctop.controller;

import cn.com.ctop.bytedance.mapper.MaterialReportMapper;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.StringUtils;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.entity.Advertiser;
import org.jeecg.modules.ctop.entity.Project;
import org.jeecg.modules.ctop.mapper.ProjectMapper;
import org.jeecg.modules.ctop.mapper.ProjectMemberMapper;
import org.jeecg.modules.ctop.service.IAdvertiserService;
import org.jeecg.modules.system.entity.SysCategory;
import org.jeecg.modules.system.entity.UserCompany;
import org.jeecg.modules.system.mapper.UserCompanyMapper;
import org.jeecg.modules.system.service.ISysCategoryService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.*;

/**
 * @Description: 广告主信息表
 * @Author: jeecg-boot
 * @Date: 2019-06-11
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告主信息表")
@RestController
@RequestMapping("/ctop/advertiser")
public class AdvertiserController {
    @Autowired
    private IAdvertiserService advertiserService;
    @Autowired
    private ISysCategoryService sysCategoryService;
    @Autowired
    private UserCompanyMapper userCompanyMapper;
    @Autowired
    private MaterialReportMapper materialReportMapper;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private ProjectMemberMapper projectMemberMapper;

    /**
     * 分页列表查询
     *
     * @param advertiser
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告主信息表-分页列表查询")
    @ApiOperation(value = "广告主信息表-分页列表查询", notes = "广告主信息表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Advertiser>> queryPageList(Advertiser advertiser,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "1000") Integer pageSize,
                                                   HttpServletRequest req) {

        Result<IPage<Advertiser>> result = new Result<IPage<Advertiser>>();
        String roleCode = materialReportMapper.getRoleCodeByUserId(advertiser.getUserId());
        String userId = advertiser.getUserId();
        if ("admin".equals(roleCode) || "kuaishouOperationManager".equals(roleCode)) {
            advertiser.setUserId(null);
        }
        QueryWrapper<Advertiser> queryWrapper = QueryGenerator.initQueryWrapper(advertiser, req.getParameterMap());
        if ("kuaishouOperationManager".equals(roleCode)) {
            QueryWrapper<UserCompany> userCompanyQueryWrapper = new QueryWrapper<>();
            userCompanyQueryWrapper.eq("user_id", userId);
            userCompanyQueryWrapper.orderByDesc("create_time");
            userCompanyQueryWrapper.last("limit 1");
            UserCompany userCompany = userCompanyMapper.selectOne(userCompanyQueryWrapper);
            queryWrapper.eq("company_id", userCompany.getCompanyId());
        }
        Page<Advertiser> page = new Page<Advertiser>(pageNo, pageSize);
        IPage<Advertiser> pageList = advertiserService.page(page, queryWrapper);
        if (pageList != null && pageList.getRecords() != null && pageList.getRecords().size() > 0) {
            List<Advertiser> list = new ArrayList<>();
            for (int i = 0; i < pageList.getRecords().size(); i++) {
                Advertiser advertiserRecord = pageList.getRecords().get(i);
                SysCategory sysCategory = sysCategoryService.getById(advertiserRecord.getIndustryId());
                if (sysCategory != null) {
                    advertiserRecord.setIndustryId(sysCategory.getName());
                }
                list.add(advertiserRecord);
            }
            pageList.setRecords(list);
        }
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param advertiser
     * @return
     */
    @AutoLog(value = "广告主信息表-添加")
    @ApiOperation(value = "广告主信息表-添加", notes = "广告主信息表-添加")
    @PostMapping(value = "/add")
    public Result<Advertiser> add(@RequestBody Advertiser advertiser) {
        Result<Advertiser> result = new Result<Advertiser>();
        try {
            String name = advertiser.getName().trim();
            QueryWrapper<Advertiser> advertiserQueryWrapper = new QueryWrapper<>();
            advertiserQueryWrapper.eq("name", name);
            advertiserQueryWrapper.last("limit 1");
            advertiserQueryWrapper.orderByDesc("create_time");
            Advertiser checkAdvertiser = advertiserService.getOne(advertiserQueryWrapper);
            if (!Check.isNull(checkAdvertiser)) {
                throw new Exception("广告主已存在");
            }

            String userId = advertiser.getUserId();
            if (!Check.isNull(userId)) {
                QueryWrapper<UserCompany> userCompanyQueryWrapper = new QueryWrapper<>();
                userCompanyQueryWrapper.eq("user_id", userId);
                userCompanyQueryWrapper.orderByDesc("create_time");
                userCompanyQueryWrapper.last("limit 1");
                UserCompany userCompany = userCompanyMapper.selectOne(userCompanyQueryWrapper);
                if (!Check.isNull(userCompany)) {
                    advertiser.setCompanyId(userCompany.getCompanyId());
                }
            }
            advertiserService.save(advertiser);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("添加失败 " + e.getMessage());
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param advertiser
     * @return
     */
    @AutoLog(value = "广告主信息表-编辑")
    @ApiOperation(value = "广告主信息表-编辑", notes = "广告主信息表-编辑")
    @PutMapping(value = "/edit")
    public Result<Advertiser> edit(@RequestBody Advertiser advertiser) {
        Result<Advertiser> result = new Result<Advertiser>();
        Advertiser advertiserEntity = advertiserService.getById(advertiser.getId());
        if (advertiserEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = advertiserService.updateById(advertiser);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告主信息表-通过id删除")
    @ApiOperation(value = "广告主信息表-通过id删除", notes = "广告主信息表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Advertiser> delete(@RequestParam(name = "id", required = true) String id) {
        Result<Advertiser> result = new Result<Advertiser>();
        Advertiser advertiser = advertiserService.getById(id);
        if (advertiser == null) {
            result.error500("未找到对应实体");
        } else {
            QueryWrapper<Project> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("advertiser_id", id);
            List<Project> projects = projectMapper.selectList(queryWrapper);
            if (!Check.isNull(projects)) {
                for (Project project : projects) {
                    Map<String, Object> map = new HashMap<>();
                    map.put("project_id", project.getId());
                    projectMemberMapper.deleteByMap(map);
                }

            }
            projectMapper.delete(queryWrapper);
            boolean ok = advertiserService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告主信息表-批量删除")
    @ApiOperation(value = "广告主信息表-批量删除", notes = "广告主信息表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<Advertiser> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<Advertiser> result = new Result<Advertiser>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.advertiserService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告主信息表-通过id查询")
    @ApiOperation(value = "广告主信息表-通过id查询", notes = "广告主信息表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Advertiser> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<Advertiser> result = new Result<>();
        Advertiser advertiser = advertiserService.getById(id);
        if (advertiser == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(advertiser);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<Advertiser> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                Advertiser advertiser = JSON.parseObject(deString, Advertiser.class);
                queryWrapper = QueryGenerator.initQueryWrapper(advertiser, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<Advertiser> pageList = advertiserService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "广告主信息表列表");
        mv.addObject(NormalExcelConstants.CLASS, Advertiser.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("广告主信息表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            // 获取上传文件对象
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<Advertiser> listAdvertisers = ExcelImportUtil.importExcel(file.getInputStream(), Advertiser.class, params);
                for (Advertiser advertiserExcel : listAdvertisers) {
                    advertiserService.save(advertiserExcel);
                }
                return Result.ok("文件导入成功！数据行数:" + listAdvertisers.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }
}
