package org.jeecg.modules.ctop.job;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.util.DateUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
public class KuaishouMaterialsLoadJob implements Job {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    static ExecutorService executorService = null;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        //增加接口的开始和结束日期
        Date startDate = DateUtils.addDay(new Date(),-1);
        Date endDate = new Date();

        //1:查询当日数据
        List<CtopOauthToken> tokens = tokenService.getTokenListByType(CtopAdConstant.PLATFORM_TYPE_KUAISHOU);

        executorService = Executors.newFixedThreadPool(10);
        tokens.forEach(token -> {
            executorService.submit(new Runnable() {
                @Override
                public void run() {
                    try {
                        //1：获取全量广告计划数据
                        kuaishouInterfaceService.getCampaignList(token,startDate,endDate);
                        //1：获取全量广告组数据
                        kuaishouInterfaceService.getGroupList(token,startDate,endDate);
                        //1：获取全量创意数据
                        kuaishouInterfaceService.getCreativeList(token,startDate,endDate);
                        //2：获取全量视频素材数据
                        kuaishouInterfaceService.getVideoList(token,startDate,endDate);
                        //3: 获取图片信息数据
                        kuaishouInterfaceService.getImageList(token,startDate,endDate);
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                    }
                }
            });
        });
        log.info("快手物料数据同步完成");
    }
}
