package cn.com.ctop.kuaishou.modules.batch.controller;


import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouCampaign;
import cn.com.ctop.kuaishou.modules.batch.entity.vo.SpendVo;
import cn.com.ctop.kuaishou.modules.batch.service.IBatchService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouCampaignService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;

@Slf4j
@Api(tags = "快手-批量工具")
@RestController
@RequestMapping("/kuaishou/batch")
public class BatchController {
    @Autowired
    private ICtopOauthTokenService oauthTokenService;
    @Autowired
    private IKuaishouInterfaceService iKuaishouInterfaceService;
    @Autowired
    private IBatchService batchService;
    @Autowired
    private IKuaiShouCampaignService kuaiShouCampaignService;


    /**
     * 获取花费信息
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "/spend")
    public Result<SpendVo> spend(Long accountId) {
        Result<SpendVo> result = new Result<>();
        try {

            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }
            SpendVo spendVo = new SpendVo();
            JSONObject fundJson = iKuaishouInterfaceService.fundGet(oauthToken);
            if (!Check.isNull(fundJson)) {
                spendVo.setBalance(fundJson.getBigDecimal("balance")); // 余额
            }
            BigDecimal cost = batchService.getCost(accountId);
            spendVo.setCost(cost);

            BigDecimal budget = batchService.getBudget(accountId);
            spendVo.setBudget(budget);
            result.setSuccess(true);
            result.setResult(spendVo);

        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }

        return result;
    }


    /**
     * 获取广告计划列表
     *
     * @param kuaiShouCampaign
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/getCampaignList")
    public Result<IPage<KuaiShouCampaign>> getCampaignList(KuaiShouCampaign kuaiShouCampaign,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<KuaiShouCampaign>> result = new Result<IPage<KuaiShouCampaign>>();
        QueryWrapper<KuaiShouCampaign> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouCampaign, req.getParameterMap());
        Page<KuaiShouCampaign> page = new Page<KuaiShouCampaign>(pageNo, pageSize);
        IPage<KuaiShouCampaign> pageList = kuaiShouCampaignService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


}
